import { Option } from './option.js'
import { range, tee } from './utils.js'

export class Stream {
  _iterator
  _index

  constructor(iterator, index = 0) {
    this._iterator = Iterator.from(iterator)
    this._index = index
  }

  get index() {
    return this._index
  }

  clone() {
    const [a, b] = tee(this._iterator)
    this._iterator = a
    return new Stream(b, this.index)
  }

  consume() {
    return Option.from(this.next().value)
  }

  *take(limit = 1) {
    for (const _index of range(limit)) {
      const next = this.next()
      if (next.done) { return }
      yield next.value
    }
  }

  drop(limit = 1) {
    while (limit > 0 && this.consume().isSome()) {
      limit -= 1
    }

    return this
  }

  peek() {
    const [a, b] = tee(this._iterator)
    this._iterator = a
    return Option.from(b.next().value)
  }

  next() {
    const result = this._iterator.next()

    if (!result.done) {
      this._index += 1
    }

    return result
  }

  [Symbol.iterator]() {
    return this
  }
}

