import { Result } from './result.js'

export class Option {
  static some(value) {
    return new Some(value)
  }

  static none() {
    return None
  }

  static from(value) {
    if (value == null) {
      return None
    } else {
      return new Some(value)
    }
  }

  isSome() {
    return this instanceof Some
  }

  isNone() {
    return this === None
  }
}

export class Some extends Option {
  #value

  constructor(value) {
    super()
    this.#value = value
    Object.freeze(this)
  }

  ap(value) {
    return this.#value(value)
  }

  filter(predicate) {
    if (predicate(this.#value)) {
      return this
    } else {
      return None
    }
  }

  andThen(fn) {
    return fn(this.#value)
  }

  map(fn) {
    return new Some(fn(this.#value))
  }

  okOr(_error) {
    return Result.ok(this.#value)
  }

  okOrElse(_fn) {
    return Result.ok(this.#value)
  }

  unwrap() {
    return this.#value
  }
}

class _None extends Option {
  constructor() {
    super()
    Object.freeze(this)
  }

  ap(_value) {
    return this
  }

  filter(_predicate) {
    return this
  }

  map(_fn) {
    return this
  }

  andThen(_fn) {
    return this
  }

  okOr(error) {
    return Result.err(error)
  }

  okOrElse(fn) {
    return Result.err(fn())
  }

  unwrap() {
    throw new UnwrapError('attempted to unwrap a None value')
  }
}

export const None = new _None()
