export const curry = (fn, arity = fn.length) => {
  return function curried(...args) {
    if (args.length >= arity) {
      return fn.apply(this, args)
    } else {
      return function(...nextArgs) {
        return curried.apply(this, args.concat(nextArgs))
      }
    }
  }
}

export function* range(start, stop, step = 1) {
  if (stop == null) {
    stop = start
    start = 0
  }

  for (let i = start; i < stop; i += step) {
    yield i
  }
}

export function tee(iterator, n = 2) {
  iterator = Iterator.from(iterator)

  function* gen(current) {
    while (true) {
      if (!current.next) {
        const { done, value } = iterator.next()
        if (done) return
        current.next = { value }
      }
      current = current.next
      yield current.value
    }
  }

  return Array(n).fill({}).map(gen)
}

